#class auto
#define PORTA_AUX_IO


/*	Hardware definitions for 12864

		Data bus:	LCD D0-7

		PB.2	D/I_
      PB.3	CS1
      PB.4	CS2
		PB.5	R/W_
		PE.1	E

*/


void MsDelay ( int iDelay )
{
	unsigned long ul0;
	ul0 = MS_TIMER;		// get current timer value
	while ( MS_TIMER < ul0 + (unsigned long) iDelay );
}


int dataaddrd,cmdaddrd,dataaddwr,cmdaddwr;

/* Low level functions */

#asm
;function requires no parameter
;
LCD_Status::
l1:
	ld hl,(cmdaddrd)
	ioe ld a,(HL)
	rla					; Check busy
	jr c,l1					; loop while busy
	ret

;function requires no parameter
;function returns one parameter (HL)
;
LCD_ReadData::
	call LCD_Status
	ld hl,(dataaddrd)
	ioe ld l,(HL)
   ld h,0
   ret


;function requires one parameter:
;@sp+2= data to write
;
LCD_WriteData::
	call LCD_Status
	ld hl,(sp+2)		  		; get value (LSB)
	ld a,l
	ld hl,(dataaddwr)
	ioe ld (HL),a
	ret

;function requires one parameter:
;@sp+2= cmd to write
;
LCD_WriteCmd::
	call LCD_Status			; Check busy
	ld hl,(sp+2)		  		; get value (LSB)
	ld a,l
	ld hl,(cmdaddwr)
	ioe ld (HL),a
	ret

#endasm





/* HD61202 support routines */

void LCD_SelSide(int side)
{
	if(side)	{
   	dataaddwr=0x2005;
   	dataaddrd=0x200D;
      cmdaddrd=0x200C;
      cmdaddwr=0x2004;
	}
	else	{
   	dataaddwr=0x2003;
   	dataaddrd=0x200B;
      cmdaddwr=0x2002;
      cmdaddrd=0x200A;
	}
}

void LCD_CalcPage(int y,int *page,int *row)
{
	*page=(y>>3);				// page = integer part of y coord /8
	*row=y-((*page)<<3);		// row = remainder
}




void LCD_init ()
{
// Use Port E bit 1 for active high I/O strobe with 15 wait-states,
#define LCD_STROBE  		0x02
#define LCD_CSREGISTER	IB1CR
#define LCD_CSSHADOW		IB1CRShadow
#define LCD_CSCONFIG		0x3C

	// Initialize Port E bit to be a normal I/O pin
	WrPortI(PEFR, &PEFRShadow, (PEFRShadow|LCD_STROBE));

	// Initialize Port E bit to be an output pin
	WrPortI(PEDDR, &PEDDRShadow, (PEDDRShadow|LCD_STROBE));

	// Initialize Port E bit to be a chip select.
	WrPortI(LCD_CSREGISTER, &LCD_CSSHADOW, LCD_CSCONFIG);

	// Set Port E bit to be clocked by PCLK/2
	WrPortI(PECR, &PECRShadow, (PECRShadow & ~0xFF));

//	MsDelay ( 1000 );				// wait for LCD to reset itself

	LCD_SelSide(1);
	LCD_WriteCmd ( '\B00111111' );			// Display on
	LCD_SelSide(0);
	LCD_WriteCmd ( '\B00111111' );			// Display on
}

/* High level functions */

void LCD_home ( void )
{
	LCD_SelSide(1);
	LCD_WriteCmd ('\B01000000');			// address 0
	LCD_WriteCmd ('\B11000000');			// display starts at address 0
	LCD_WriteCmd ('\B00111111' );			// Display on
	LCD_SelSide(0);
	LCD_WriteCmd ('\B01000000');			// address 0
	LCD_WriteCmd ('\B11000000');			// display starts at address 0
	LCD_WriteCmd ('\B00111111' );			// Display on
}

void LCD_fill(unsigned char pattern)
{
int i,page;
	LCD_home();										// address 0
	for(page=0;page<8;page++) {
		LCD_SelSide(0);							// select left
		LCD_WriteCmd (0xB8+page);				// set page address
		for(i=0;i<64;i++)
			LCD_WriteData(pattern);				// fill with pattern
		LCD_SelSide(1);							// select right
		LCD_WriteCmd (0xB8+page);				// set page address
		for(i=0;i<64;i++)
			LCD_WriteData(pattern);				// fill with pattern
	}
}

#define LCD_clear() LCD_fill(0)


const static unsigned char font5x7[] = {
0x00,0x00,0x00,0x00,0x00,	// 0x20 ' '
0x00,0x00,0xBE,0x00,0x00,	// 0x21 '!'
0x00,0x0E,0x00,0x0E,0x00,	// 0x22 '"'
0x28,0xFE,0x28,0xFE,0x28,	// 0x23 '#'
0x48,0x54,0xFE,0x54,0x24,	// 0x24 '$'
0x46,0x26,0x10,0xC8,0xC4,	// 0x25 '%'
0x6C,0x92,0xAA,0x44,0xA0,	// 0x26 '&'
0x00,0x0A,0x06,0x00,0x00,	// 0x27 '''
0x00,0x38,0x44,0x82,0x00,	// 0x28 '('
0x00,0x82,0x44,0x38,0x00,	// 0x29 ')'
0x28,0x10,0x7C,0x10,0x28,	// 0x2A '*'
0x10,0x10,0x7C,0x10,0x10,	// 0x2B '+'
0x00,0xA0,0x60,0x00,0x00,	// 0x2C ','
0x10,0x10,0x10,0x10,0x10,	// 0x2D '-'
0x00,0xC0,0xC0,0x00,0x00,	// 0x2E '.'
0x40,0x20,0x10,0x08,0x04,	// 0x2F '/'
0x7C,0xA2,0x92,0x8A,0x7C,	// 0x30 '0'
0x00,0x84,0xFE,0x80,0x00,	// 0x31 '1'
0x84,0xC2,0xA2,0x92,0x8C,	// 0x32 '2'
0x42,0x82,0x8A,0x96,0x62,	// 0x33 '3'
0x30,0x28,0x24,0xFE,0x20,	// 0x34 '4'
0x4E,0x8A,0x8A,0x8A,0x72,	// 0x35 '5'
0x78,0x94,0x92,0x92,0x60,	// 0x36 '6'
0x02,0xE2,0x12,0x0A,0x06,	// 0x37 '7'
0x6C,0x92,0x92,0x92,0x6C,	// 0x38 '8'
0x0C,0x92,0x92,0x52,0x3C,	// 0x39 '9'
0x00,0x6C,0x6C,0x00,0x00,	// 0x3A ':'
0x00,0xAC,0x6C,0x00,0x00,	// 0x3B ';'
0x10,0x28,0x44,0x82,0x00,	// 0x3C '<'
0x28,0x28,0x28,0x28,0x28,	// 0x3D '='
0x82,0x44,0x28,0x10,0x00,	// 0x3E '>'
0x04,0x02,0xA2,0x12,0x0C,	// 0x3F '?'
0x64,0x92,0xF2,0x82,0x7C,	// 0x40 '@'
0xFC,0x22,0x22,0x22,0xFC,	// 0x41 'A'
0xFE,0x92,0x92,0x92,0x6C,	// 0x42 'B'
0x7C,0x82,0x82,0x82,0x44,	// 0x43 'C'
0xFE,0x82,0x82,0x82,0x7C,	// 0x44 'D'
0xFE,0x92,0x92,0x92,0x82,	// 0x45 'E'
0xFE,0x12,0x12,0x12,0x02,	// 0x46 'F'
0x7C,0x82,0x92,0x92,0xF4,	// 0x47 'G'
0xFE,0x10,0x10,0x10,0xFE,	// 0x48 'H'
0x00,0x82,0xFE,0x82,0x00,	// 0x49 'I'
0x40,0x80,0x82,0x7E,0x02,	// 0x4A 'J'
0xFE,0x10,0x28,0x44,0x82,	// 0x4B 'K'
0xFE,0x80,0x80,0x80,0x80,	// 0x4C 'L'
0xFE,0x02,0x04,0x02,0xFE,	// 0x4D 'M'
0xFE,0x08,0x10,0x20,0xFE,	// 0x4E 'N'
0x7C,0x82,0x82,0x82,0x7C,	// 0x4F 'O'
0xFE,0x12,0x12,0x12,0x0C,	// 0x50 'P'
0x7C,0x82,0xA2,0x42,0xBC,	// 0x51 'Q'
0xFE,0x12,0x32,0x52,0x8C,	// 0x52 'R'
0x8C,0x92,0x92,0x92,0x62,	// 0x53 'S'
0x02,0x02,0xFE,0x02,0x02,	// 0x54 'T'
0x7E,0x80,0x80,0x80,0x7E,	// 0x55 'U'
0x3E,0x40,0x80,0x40,0x3E,	// 0x56 'V'
0x7E,0x80,0x70,0x80,0x7E,	// 0x57 'W'
0xC6,0x28,0x10,0x28,0xC6,	// 0x58 'X'
0x0E,0x10,0xE0,0x10,0x0E,	// 0x59 'Y'
0xC2,0xA2,0x92,0x8A,0x86,	// 0x5A 'Z'
0x00,0xFE,0x82,0x82,0x00,	// 0x5B '['
0x2A,0x2C,0xF8,0x2C,0x2A,	// 0x5C '\'
0x00,0x82,0x82,0xFE,0x00,	// 0x5D ']'
0x08,0x04,0x02,0x04,0x08,	// 0x5E '^'
0x80,0x80,0x80,0x80,0x80,	// 0x5F '_'
0x00,0x02,0x04,0x08,0x00,	// 0x60 '`'
0x40,0xA8,0xA8,0xA8,0xF0,	// 0x61 'a'
0xFE,0x90,0x88,0x88,0x70,	// 0x62 'b'
0x70,0x88,0x88,0x88,0x40,	// 0x63 'c'
0x70,0x88,0x88,0x90,0xFE,	// 0x64 'd'
0x70,0xA8,0xA8,0xA8,0x30,	// 0x65 'e'
0x10,0xFC,0x12,0x02,0x04,	// 0x66 'f'
0x0C,0x92,0x92,0x92,0x7E,	// 0x67 'g'
0xFE,0x10,0x08,0x08,0xF0,	// 0x68 'h'
0x00,0x88,0xFA,0x80,0x00,	// 0x69 'i'
0x40,0x80,0x84,0x7E,0x00,	// 0x6A 'j'
0xFE,0x20,0x50,0x88,0x00,	// 0x6B 'k'
0x00,0x82,0xFE,0x80,0x00,	// 0x6C 'l'
0xF8,0x08,0x30,0x08,0xF0,	// 0x6D 'm'
0xF8,0x10,0x08,0x08,0xF0,	// 0x6E 'n'
0x70,0x88,0x88,0x88,0x70,	// 0x6F 'o'
0xF8,0x28,0x28,0x28,0x10,	// 0x70 'p'
0x10,0x28,0x28,0x30,0xF8,	// 0x71 'q'
0xF8,0x10,0x08,0x08,0x10,	// 0x72 'r'
0x90,0xA8,0xA8,0xA8,0x48,	// 0x73 's'
0x04,0x7E,0x84,0x80,0x40,	// 0x74 't'
0x78,0x80,0x80,0x40,0xF8,	// 0x75 'u'
0x38,0x40,0x80,0x40,0x38,	// 0x76 'v'
0x78,0x80,0x60,0x80,0x78,	// 0x77 'w'
0x88,0x50,0x20,0x50,0x88,	// 0x78 'x'
0x18,0xA0,0xA0,0xA0,0x78,	// 0x79 'y'
0x88,0xC8,0xA8,0x98,0x88,	// 0x7A 'z'
0x00,0x10,0x6C,0x82,0x00,	// 0x7B '{'
0x00,0x00,0xFE,0x00,0x00,	// 0x7C '|'
0x00,0x82,0x6C,0x10,0x00,	// 0x7D '}'
0x10,0x10,0x54,0x38,0x10,	// 0x7E '~'
0x10,0x38,0x54,0x10,0x10,	// 0x7F ' '
};


void LCD_putchr ( char chr )
{
int i;
char *addr;

	if((chr>0x20) && (chr<0x7F)){
		addr=&font5x7[5*(chr-0x20)];
		i=5;
      while(i--)
			LCD_WriteData(*addr++);				// send character
		LCD_WriteData(0);	 				// send intercharacter separator
   }
}



void LCD_printat (unsigned int row, unsigned int col, char *ptr)
{
int x;
	LCD_home();										// just in case
	do {
		if(col>19){
			col=0;
			row++;
		}
		x=(col>9)?6*(col-10):6*col;				// correct x address if col>9
		if(col>9)										// 0-9 => side 0; 10>20 => side 1
			LCD_SelSide(1);
		else {
			LCD_SelSide(0);
			x+=4;											// 4 pixels offset at both sides
		}
		LCD_WriteCmd (0xB8+row);					// set page address
		LCD_WriteCmd (0x40+x);						// address = 6*col or 6*col-10 (4 pixels offset)
		LCD_putchr (*ptr++);
		col++;
	} 	while (*ptr);
}
void LCD_scroll1()
{
int i,page,data;

	for(page=2;page<8;page++) {				// leave page 0 alone (datetime)
		for(i=0;i<64;i++){
			LCD_WriteCmd (0xB8+page);			// set page address
			LCD_ReadData();						// dummy read before reading
			data=LCD_ReadData();					// get data for each page
			if(page==7){
				LCD_WriteCmd (0x40+i);			// set address again (reading increments counter)
				LCD_WriteData(0);					// clear last row (page 7)
			}
			LCD_WriteCmd (0x40+i);				// set address again (reading increments counter)
			LCD_WriteCmd (0xB7+page);			// set page address at prior page
			LCD_WriteData(data);					// and move it
		}
	}
}
void LCD_scroll()
{
	LCD_home();										// address 0,0
	LCD_SelSide(1);								// select right
	LCD_scroll1();									// scroll
	LCD_SelSide(0);								// select left
	LCD_scroll1();									// scroll
}




/* MAIN PROGRAM */

main()
{
int i;
unsigned long t;


	clockDoublerOff();

	LCD_init();
   LCD_clear();

	LCD_printat(0,0,"Hola, funciona...");
}












